package com.company.onboarding.view.user;

import com.company.onboarding.entity.User;
import com.company.onboarding.view.main.MainView;
import com.vaadin.flow.component.ClickEvent;
import com.vaadin.flow.component.html.Image;
import com.vaadin.flow.data.renderer.ComponentRenderer;
import com.vaadin.flow.data.renderer.Renderer;
import com.vaadin.flow.router.Route;
import com.vaadin.flow.server.StreamResource;
import io.jmix.core.DataManager;
import io.jmix.core.FileRef;
import io.jmix.core.FileStorage;
import io.jmix.flowui.Notifications;
import io.jmix.flowui.UiComponents;
import io.jmix.flowui.component.grid.DataGrid;
import io.jmix.flowui.kit.component.button.JmixButton;
import io.jmix.flowui.view.*;
import io.jmix.securitydata.entity.RoleAssignmentEntity;
import org.springframework.beans.factory.annotation.Autowired;

@Route(value = "users", layout = MainView.class)
@ViewController("User.list")
@ViewDescriptor("user-list-view.xml")
@LookupComponent("usersDataGrid")
@DialogMode(width = "50em", height = "37.5em")
public class UserListView extends StandardListView<User> {

    @Autowired
    private UiComponents uiComponents;

    @Autowired
    private FileStorage fileStorage;
    @Autowired
    private DataManager dataManager;
    @ViewComponent
    private DataGrid<User> usersDataGrid;
    @Autowired
    private Notifications notifications;

    @Supply(to = "usersDataGrid.picture", subject = "renderer")
    private Renderer<User> usersDataGridPictureRenderer() {
        return new ComponentRenderer<>(user -> {
            FileRef fileRef = user.getPicture();
            if (fileRef != null) {
                Image image = uiComponents.create(Image.class);
                image.setWidth("30px");
                image.setHeight("30px");
                StreamResource streamResource = new StreamResource(
                        fileRef.getFileName(),
                        () -> fileStorage.openStream(fileRef));
                image.setSrc(streamResource);
                image.setClassName("user-picture");

                return image;
            } else {
                return null;
            }
        });
    }

    @Subscribe(id = "generateHrManagerGroupBtn", subject = "clickListener")
    public void onGenerateGroupsBtnClick(final ClickEvent<JmixButton> event) {
        // Load all Alice's role assignments
        dataManager.load(RoleAssignmentEntity.class)
                .query("e.username = ?1", "alice")
                .list()
                .forEach(prototypeAssignment -> {
                    // And save them for hr-manager-group user (which actually doesn't exist)
                    RoleAssignmentEntity groupAssignment = dataManager.create(RoleAssignmentEntity.class);
                    groupAssignment.setUsername("hr-manager-group");
                    groupAssignment.setRoleType(prototypeAssignment.getRoleType());
                    groupAssignment.setRoleCode(prototypeAssignment.getRoleCode());
                    dataManager.save(groupAssignment);
                });
        notifications.show("Done");
    }

    @Subscribe(id = "assignHrManagerRolesBtn", subject = "clickListener")
    public void onAssignHrManagerRolesBtnClick(final ClickEvent<JmixButton> event) {
        User user = usersDataGrid.getSingleSelectedItem();
        if (user != null) {
            dataManager.load(RoleAssignmentEntity.class)
                    .query("e.username = ?1", "hr-manager-group")
                    .list()
                    .forEach(prototypeAssignment -> {
                        RoleAssignmentEntity groupAssignment = dataManager.create(RoleAssignmentEntity.class);
                        groupAssignment.setUsername(user.getUsername());
                        groupAssignment.setRoleType(prototypeAssignment.getRoleType());
                        groupAssignment.setRoleCode(prototypeAssignment.getRoleCode());
                        dataManager.save(groupAssignment);
                    });
            notifications.show("Done");
        } else {
            notifications.show("Select a user");
        }
    }
}